@@ -192,6 +192,7 @@ tracker_db_journal_error_quark (void)
 gboolean
 tracker_db_journal_init (const gchar *filename)
 {
+	gchar *directory;
 	struct stat st;
 	int flags;
 	int mode;
@@ -215,6 +216,23 @@ tracker_db_journal_init (const gchar *filename)
 		                                            NULL);
 	}
 
+	directory = g_path_get_dirname (writer.journal_filename);
+	if (g_strcmp0 (directory, ".")) {
+		mode = S_IRWXU | S_IRWXG | S_IRWXO;
+		if (g_mkdir_with_parents (directory, mode)) {
+			g_critical ("tracker data directory does not exist and "
+				    "could not be created: %s",
+				    g_strerror (errno));
+
+			g_free (directory);
+			g_free (writer.journal_filename);
+			writer.journal_filename = NULL;
+
+			return FALSE;
+		}
+	}
+	g_free (directory);
+
 	mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP;
 	flags = O_WRONLY | O_APPEND | O_CREAT;
 	writer.journal = g_open (writer.journal_filename, flags, mode);
@@ -294,8 +312,8 @@ tracker_db_journal_get_size (void)
 const gchar *
 tracker_db_journal_get_filename (void)
 {
-	g_return_val_if_fail (writer.journal > 0, FALSE);
-
+	/* Journal doesn't have to be open to get the filename, for example when
+	 * the file didn't exist and it was attempted opened in only read mode. */
 	return (const gchar*) writer.journal_filename;
 }
 
@@ -661,13 +679,23 @@ gboolean
 tracker_db_journal_reader_verify_last (GError **error)
 {
 	guint32 entry_size_check;
+	gboolean success = FALSE;
 
-	g_return_val_if_fail (reader.file != NULL, FALSE);
+	if (tracker_db_journal_reader_init (NULL)) {
+		entry_size_check = read_uint32 (reader.end - 4);
 
-	entry_size_check = read_uint32 (reader.end - 4);
-	reader.current = reader.end - entry_size_check;
+		if (reader.end - entry_size_check < reader.current) {
+			g_set_error (error, TRACKER_DB_JOURNAL_ERROR, 0, 
+			             "Damaged journal entry at end of journal");
+			return FALSE;
+		}
 
-	return tracker_db_journal_reader_next (error);
+		reader.current = reader.end - entry_size_check;
+		success = tracker_db_journal_reader_next (NULL);
+		tracker_db_journal_reader_shutdown ();
+	}
+
+	return success;
 }
 
 gboolean
