@@ -21,12 +21,15 @@
 
 #include <libtracker-common/tracker-dbus.h>
 #include <libtracker-common/tracker-file-utils.h>
+#include <libtracker-common/tracker-log.h>
+#include <libtracker-common/tracker-utils.h>
 
 #include "tracker-crawler.h"
 #include "tracker-marshal.h"
 #include "tracker-miner-fs.h"
 #include "tracker-monitor.h"
 #include "tracker-utils.h"
+#include "tracker-thumbnailer.h"
 
 /**
  * SECTION:tracker-miner-fs
@@ -60,11 +63,12 @@ typedef struct {
 typedef struct {
 	GMainLoop *main_loop;
 	gchar *iri;
+	gchar *mime;
+	gboolean get_mime;
 } SparqlQueryData;
 
 typedef struct {
 	GMainLoop *main_loop;
-	gint       level;
 	GString   *sparql;
 	const gchar *source_uri;
 	const gchar *uri;
@@ -113,6 +117,10 @@ struct TrackerMinerFSPrivate {
 	guint           directories_ignored;
 	guint           files_found;
 	guint           files_ignored;
+
+	guint           total_files_processed;
+	guint           total_files_notified;
+	guint           total_files_notified_error;
 };
 
 enum {
@@ -210,7 +218,7 @@ static void           crawl_directories_stop       (TrackerMinerFS *fs);
 
 static void           item_queue_handlers_set_up   (TrackerMinerFS *fs);
 
-static void           item_update_uri_recursively (TrackerMinerFS    *fs,
+static void           item_update_children_uri    (TrackerMinerFS    *fs,
                                                    RecursiveMoveData *data,
                                                    const gchar       *source_uri,
                                                    const gchar       *uri);
@@ -771,6 +779,10 @@ process_print_stats (TrackerMinerFS *fs)
 		           fs->private->total_files_ignored);
 		g_message ("Total monitors    : %d",
 		           tracker_monitor_get_count (fs->private->monitor));
+		g_message ("Total files processed : %d (%d notified, %d with error)\n",
+			   fs->private->total_files_processed,
+			   fs->private->total_files_notified,
+			   fs->private->total_files_notified_error);
 		g_message ("--------------------------------------------------\n");
 	}
 }
@@ -904,9 +916,11 @@ sparql_query_cb (GObject      *object,
 
 		val = g_ptr_array_index (query_results, 0);
 		data->iri = g_strdup (val[0]);
+		if (data->get_mime)
+			data->mime = g_strdup (val[1]);
 	}
 
-        g_main_loop_quit (data->main_loop);
+	g_main_loop_quit (data->main_loop);
 }
 
 static void
@@ -974,29 +988,35 @@ item_add_or_update (TrackerMinerFS *fs,
 	               &processing);
 
 	if (!processing) {
+		gchar *uri;
+
+		uri = g_file_get_uri (file);
+
 		/* Re-fetch data, since it might have been
 		 * removed in broken implementations
 		 */
 		data = process_data_find (fs, file);
 
-		if (!data) {
-			gchar *uri;
+		g_message ("%s refused to process '%s'", G_OBJECT_TYPE_NAME (fs), uri);
 
-			uri = g_file_get_uri (file);
+		if (!data) {
 			g_critical ("%s has returned FALSE in ::process-file for '%s', "
 			            "but it seems that this file has been processed through "
 			            "tracker_miner_fs_notify_file(), this is an "
 			            "implementation error", G_OBJECT_TYPE_NAME (fs), uri);
-			g_free (uri);
 		} else {
 			priv->processing_pool = g_list_remove (priv->processing_pool, data);
 			process_data_free (data);
 		}
+
+		g_free (uri);
 	} else {
 		guint length;
 
 		length = g_list_length (priv->processing_pool);
 
+		fs->private->total_files_processed++;
+
 		if (length >= priv->pool_limit) {
 			retval = FALSE;
 		}
@@ -1012,14 +1032,22 @@ item_add_or_update (TrackerMinerFS *fs,
 static gboolean
 item_query_exists (TrackerMinerFS  *miner,
                    GFile           *file,
-                   gchar          **iri)
+                   gchar          **iri,
+                   gchar          **mime)
 {
 	gboolean   result;
 	gchar     *sparql, *uri;
 	SparqlQueryData data = { 0 };
 
+	data.get_mime = (mime != NULL);
+
 	uri = g_file_get_uri (file);
-	sparql = g_strdup_printf ("SELECT ?s WHERE { ?s nie:url '%s' }", uri);
+
+	if (data.get_mime) {
+		sparql = g_strdup_printf ("SELECT ?s ?m WHERE { ?s nie:url '%s' . OPTIONAL { ?s nie:mimeType ?m } }", uri);
+	} else {
+		sparql = g_strdup_printf ("SELECT ?s WHERE { ?s nie:url '%s' }", uri);
+	}
 
 	data.main_loop = g_main_loop_new (NULL, FALSE);
 
@@ -1040,6 +1068,12 @@ item_query_exists (TrackerMinerFS  *miner,
 		g_free (data.iri);
 	}
 
+	if (mime) {
+		*mime = data.mime;
+	} else {
+		g_free (data.mime);
+	}
+
 	g_free (sparql);
 	g_free (uri);
 
@@ -1052,6 +1086,7 @@ item_remove (TrackerMinerFS *fs,
 {
 	GString *sparql;
 	gchar *uri, *slash_uri;
+	gchar *mime = NULL;
 	ProcessData *data;
 
 	uri = g_file_get_uri (file);
@@ -1059,12 +1094,17 @@ item_remove (TrackerMinerFS *fs,
 	g_debug ("Removing item: '%s' (Deleted from filesystem)",
 	         uri);
 
-	if (!item_query_exists (fs, file, NULL)) {
+	if (!item_query_exists (fs, file, NULL, &mime)) {
 		g_debug ("  File does not exist anyway (uri:'%s')", uri);
 		g_free (uri);
+		g_free (mime);
 		return TRUE;
 	}
 
+	tracker_thumbnailer_remove_add (uri, mime);
+
+	g_free (mime);
+
 	if (!g_str_has_suffix (uri, "/")) {
 		slash_uri = g_strconcat (uri, "/", NULL);
 	} else {
@@ -1151,10 +1191,10 @@ item_writeback (TrackerMinerFS *fs,
 
 	g_debug ("Updating item: '%s' (Writeback event)", uri);
 
-	if (!item_query_exists (fs, working_file, NULL)) {
+	/* if (!item_query_exists (fs, working_file, NULL, NULL)) {
 		g_debug ("  File does not exist anyway (uri:'%s')", uri);
 		return TRUE;
-	}
+	} */
 
 	cancellable = g_cancellable_new ();
 	sparql = tracker_sparql_builder_new_update ();
@@ -1203,11 +1243,10 @@ item_writeback (TrackerMinerFS *fs,
 }
 
 static void
-item_update_uri_recursively_cb (GObject      *object,
-                                GAsyncResult *result,
-                                gpointer      user_data)
+item_update_children_uri_cb (GObject      *object,
+                             GAsyncResult *result,
+                             gpointer      user_data)
 {
-	TrackerMinerFS *fs = TRACKER_MINER_FS (object);
 	RecursiveMoveData *data = user_data;
 	GError *error = NULL;
 
@@ -1216,69 +1255,77 @@ item_update_uri_recursively_cb (GObject      *object,
 	if (error) {
 		g_critical ("Could not query children: %s", error->message);
 		g_error_free (error);
-	} else {
-		if (query_results) {
-			gint i;
+	} else if (query_results) {
+		gint i;
 
-			for (i = 0; i < query_results->len; i++) {
-				gchar **child_source_uri, *child_uri;
+		for (i = 0; i < query_results->len; i++) {
+			const gchar *child_source_uri, *child_mime, *child_urn;
+			gchar *child_uri;
+			GStrv row;
 
-				child_source_uri = g_ptr_array_index (query_results, i);
+			row = g_ptr_array_index (query_results, i);
+			child_urn = row[0];
+			child_source_uri = row[1];
+			child_mime = row[2];
 
-				if (!g_str_has_prefix (*child_source_uri, data->source_uri)) {
-					g_warning ("Child URI '%s' does not start with parent URI '%s'",
-					           *child_source_uri,
-					           data->source_uri);
-					continue;
-				}
-
-				child_uri = g_strdup_printf ("%s%s", data->uri, *child_source_uri + strlen (data->source_uri));
-
-				item_update_uri_recursively (fs, data, *child_source_uri, child_uri);
-
-				g_free (child_uri);
+			if (!g_str_has_prefix (child_source_uri, data->source_uri)) {
+				g_warning ("Child URI '%s' does not start with parent URI '%s'",
+					   child_source_uri,
+					   data->source_uri);
+				continue;
 			}
+
+			child_uri = g_strdup_printf ("%s%s", data->uri, child_source_uri + strlen (data->source_uri));
+
+			g_string_append_printf (data->sparql,
+			                        "DELETE FROM <%s> { "
+			                        "  <%s> nie:url ?u "
+			                        "} WHERE { "
+			                        "  <%s> nie:url ?u "
+			                        "} ",
+			                        child_urn, child_urn, child_urn);
+
+			g_string_append_printf (data->sparql,
+			                        "INSERT INTO <%s> {"
+			                        "  <%s> nie:url '%s' "
+			                        "} ",
+			                        child_urn, child_urn, child_uri);
+
+			tracker_thumbnailer_move_add (child_source_uri, child_mime, child_uri);
+
+			g_free (child_uri);
 		}
 	}
 
-	data->level--;
-
-	g_assert (data->level >= 0);
-
-	if (data->level == 0) {
-		g_main_loop_quit (data->main_loop);
-	}
+	g_main_loop_quit (data->main_loop);
 }
 
 static void
-item_update_uri_recursively (TrackerMinerFS    *fs,
-                             RecursiveMoveData *move_data,
-                             const gchar       *source_uri,
-                             const gchar       *uri)
+item_update_children_uri (TrackerMinerFS    *fs,
+                          RecursiveMoveData *move_data,
+                          const gchar       *source_uri,
+                          const gchar       *uri)
 {
-	gchar *sparql;
+	gchar *slash_uri, *sparql;
 
-	move_data->level++;
+	slash_uri = g_strconcat (source_uri, "/", NULL);
 
-	g_string_append_printf (move_data->sparql,
-	                        "INSERT { "
-	                        "  ?u nie:url '%s' "
-	                        "} WHERE { "
-	                        "  ?u nie:url '%s' "
-	                        "} ",
-	                        uri, source_uri);
-
-	sparql = g_strdup_printf ("SELECT ?child WHERE { "
-	                          "  ?child nfo:belongsToContainer ?c . "
-	                          "  ?c nie:url '%s' "
-	                          "}",
-                                  source_uri);
+	sparql = g_strdup_printf ("SELECT ?child ?url ?m WHERE { "
+				  "  ?child nie:url ?url . "
+				  "  OPTIONAL { "
+				  "    ?child nie:mimeType ?m "
+				  "  } . "
+				  "  FILTER (fn:starts-with (?url, \"%s\")) "
+				  "}",
+				  slash_uri);
 
 	tracker_miner_execute_sparql (TRACKER_MINER (fs),
 	                              sparql,
 	                              NULL,
-	                              item_update_uri_recursively_cb,
+	                              item_update_children_uri_cb,
 	                              move_data);
+
+	g_free (slash_uri);
 	g_free (sparql);
 }
 
@@ -1298,7 +1345,7 @@ item_move (TrackerMinerFS *fs,
 	source_uri = g_file_get_uri (source_file);
 
 	/* Get 'source' ID */
-	if (!item_query_exists (fs, source_file, &source_iri)) {
+	if (!item_query_exists (fs, source_file, &source_iri, NULL)) {
 		gboolean retval;
 
 		g_message ("Source file '%s' not found in store to move, indexing '%s' from scratch", source_uri, uri);
@@ -1312,7 +1359,8 @@ item_move (TrackerMinerFS *fs,
 	}
 
 	file_info = g_file_query_info (file,
-	                               G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
+	                               G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
+	                               G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
 	                               G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
 	                               NULL, NULL);
 
@@ -1332,15 +1380,30 @@ item_move (TrackerMinerFS *fs,
 	         source_uri,
 	         uri);
 
+	tracker_thumbnailer_move_add (source_uri, 
+	                              g_file_info_get_content_type (file_info),
+	                              uri);
+
 	sparql = g_string_new ("");
 
+	/* Delete destination item from store if any */
+	g_string_append_printf (sparql,
+				"DELETE { "
+	                        "  ?urn a rdfs:Resource "
+	                        "} WHERE {"
+	                        "  ?urn nie:url \"%s\" "
+				"}",
+				uri);
+
 	g_string_append_printf (sparql,
 	                        "DELETE FROM <%s> { "
 	                        "  <%s> nfo:fileName ?f ; "
-	                        "       nie:url ?u "
+	                        "       nie:url ?u ; "
+	                        "       nie:isStoredAs ?s "
 	                        "} WHERE { "
 	                        "  <%s> nfo:fileName ?f ; "
-	                        "       nie:url ?u "
+	                        "       nie:url ?u ; "
+	                        "       nie:isStoredAs ?s "
 	                        "} ",
 	                        source_iri, source_iri, source_iri);
 
@@ -1349,20 +1412,21 @@ item_move (TrackerMinerFS *fs,
 	g_string_append_printf (sparql,
 	                        "INSERT INTO <%s> {"
 	                        "  <%s> nfo:fileName '%s' ; "
-	                        "       nie:url '%s' "
+	                        "       nie:url '%s' ; "
+	                        "       nie:isStoredAs <%s> "
 	                        "} ",
 	                        source_iri, source_iri,
-	                        escaped_filename, uri);
+	                        escaped_filename, uri,
+	                        source_iri);
 
 	g_free (escaped_filename);
 
 	move_data.main_loop = g_main_loop_new (NULL, FALSE);
-	move_data.level = 0;
 	move_data.sparql = sparql;
 	move_data.source_uri = source_uri;
 	move_data.uri = uri;
 
-	item_update_uri_recursively (fs, &move_data, source_uri, uri);
+	item_update_children_uri (fs, &move_data, source_uri, uri);
 
 	g_main_loop_run (move_data.main_loop);
 
@@ -1459,7 +1523,9 @@ item_queue_get_next_file (TrackerMinerFS  *fs,
 }
 
 static gdouble
-item_queue_get_progress (TrackerMinerFS *fs)
+item_queue_get_progress (TrackerMinerFS *fs,
+                         guint          *n_items_processed,
+                         guint          *n_items_remaining)
 {
 	guint items_to_process = 0;
 	guint items_total = 0;
@@ -1472,6 +1538,14 @@ item_queue_get_progress (TrackerMinerFS *fs)
 	items_total += fs->private->total_directories_found;
 	items_total += fs->private->total_files_found;
 
+	if (n_items_processed) {
+		*n_items_processed = items_total - items_to_process;
+	}
+
+	if (n_items_remaining) {
+		*n_items_remaining = items_to_process;
+	}
+
 	if (items_to_process == 0 && items_total > 0) {
 		return 0.0;
 	}
@@ -1516,8 +1590,42 @@ item_queue_handlers_cb (gpointer user_data)
 	g_get_current_time (&time_now);
 
 	if ((time_now.tv_sec - time_last.tv_sec) >= 1) {
+		guint items_processed, items_remaining;
+		gdouble progress_now;
+		static gdouble progress_last = 0.0;
+
 		time_last = time_now;
-		g_object_set (fs, "progress", item_queue_get_progress (fs), NULL);
+
+		/* Update progress */
+		progress_now = item_queue_get_progress (fs,
+		                                        &items_processed,
+		                                        &items_remaining);
+		g_object_set (fs, "progress", progress_now, NULL);
+
+		/* Only log estimated remaining time on each 1% change */
+		if ((gint) (progress_last * 100) != (gint) (progress_now * 100)) {
+			gchar *str1, *str2;
+			gdouble seconds_elapsed;
+
+			progress_last = progress_now;
+
+			/* Log estimated remaining time */
+			seconds_elapsed = g_timer_elapsed (fs->private->timer, NULL);
+			str1 = tracker_seconds_estimate_to_string (seconds_elapsed,
+			                                           TRUE,
+			                                           items_processed,
+			                                           items_remaining);
+			str2 = tracker_seconds_to_string (seconds_elapsed, TRUE);
+
+			tracker_info ("Processed %d/%d, estimated %s left, %s elapsed",
+			              items_processed,
+			              items_processed + items_remaining,
+			              str1,
+			              str2);
+
+			g_free (str2);
+			g_free (str1);
+		}
 	}
 
 	/* Handle queues */
@@ -1529,6 +1637,7 @@ item_queue_handlers_cb (gpointer user_data)
 			process_stop (fs);
 		}
 
+		tracker_thumbnailer_send ();
 		/* No more files left to process */
 		keep_processing = FALSE;
 		break;
@@ -1666,6 +1775,7 @@ should_change_index_for_file (TrackerMinerFS *fs,
 	                         t.tm_sec,
 	                         uri);
 
+	data.get_mime = FALSE;
 	data.main_loop = g_main_loop_new (NULL, FALSE);
 
 	tracker_miner_execute_sparql (TRACKER_MINER (fs),
@@ -1679,6 +1789,7 @@ should_change_index_for_file (TrackerMinerFS *fs,
 
 	g_main_loop_unref (data.main_loop);
 	g_free (data.iri);
+	g_free (data.mime);
 	g_free (query);
 	g_free (uri);
 
@@ -1870,7 +1981,7 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 		path = g_file_get_path (file);
 		other_path = g_file_get_path (other_file);
 
-		source_stored = item_query_exists (fs, file, NULL);
+		source_stored = item_query_exists (fs, file, NULL, NULL);
 		should_process_other = should_check_file (fs, other_file, is_directory);
 
 		g_debug ("%s:'%s'->'%s':%s (%s) (move monitor event or user request)",
@@ -2163,6 +2274,42 @@ tracker_miner_fs_add_directory (TrackerMinerFS *fs,
 	crawl_directories_start (fs);
 }
 
+/**
+ * tracker_miner_fs_add_file:
+ * @fs: a #TrackerMinerFS
+ * @file: #GFile for the file to inspect
+ *
+ * Tells the filesystem miner to inspect a file.
+ **/
+void
+tracker_miner_fs_add_file (TrackerMinerFS *fs,
+                           GFile          *file)
+{
+	gboolean should_process;
+	gchar *path;
+
+	g_return_if_fail (TRACKER_IS_MINER_FS (fs));
+	g_return_if_fail (G_IS_FILE (file));
+
+	should_process = should_check_file (fs, file, FALSE);
+
+	path = g_file_get_path (file);
+
+	g_debug ("%s:'%s' (FILE) (requested by application)",
+	         should_process ? "Found " : "Ignored",
+	         path);
+
+	if (should_process) {
+		g_queue_push_tail (fs->private->items_updated,
+		                   g_object_ref (file));
+
+		item_queue_handlers_set_up (fs);
+	}
+
+	g_free (path);
+}
+
+
 static void
 check_files_removal (GQueue *queue,
                      GFile  *parent)
@@ -2336,6 +2483,12 @@ tracker_miner_fs_notify_file (TrackerMinerFS *fs,
 	g_return_if_fail (TRACKER_IS_MINER_FS (fs));
 	g_return_if_fail (G_IS_FILE (file));
 
+	fs->private->total_files_notified++;
+
+	if (error) {
+		fs->private->total_files_notified_error++;
+	}
+
 	data = process_data_find (fs, file);
 
 	if (!data) {
