@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
  *
@@ -102,15 +101,15 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 	 **/
 	signals [MINER_PROGRESS] =
 		g_signal_new ("miner-progress",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (TrackerMinerManagerClass, miner_progress),
-			      NULL, NULL,
-			      tracker_marshal_VOID__STRING_STRING_DOUBLE,
-			      G_TYPE_NONE, 3,
-			      G_TYPE_STRING,
-			      G_TYPE_STRING,
-			      G_TYPE_DOUBLE);
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (TrackerMinerManagerClass, miner_progress),
+		              NULL, NULL,
+		              tracker_marshal_VOID__STRING_STRING_DOUBLE,
+		              G_TYPE_NONE, 3,
+		              G_TYPE_STRING,
+		              G_TYPE_STRING,
+		              G_TYPE_DOUBLE);
 	/**
 	 * TrackerMinerManager::miner-paused
 	 * @manager: the #TrackerMinerManager
@@ -121,13 +120,13 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 	 **/
 	signals [MINER_PAUSED] =
 		g_signal_new ("miner-paused",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (TrackerMinerManagerClass, miner_paused),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1,
-			      G_TYPE_STRING);
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (TrackerMinerManagerClass, miner_paused),
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__STRING,
+		              G_TYPE_NONE, 1,
+		              G_TYPE_STRING);
 	/**
 	 * TrackerMinerManager::miner-resumed
 	 * @manager: the #TrackerMinerManager
@@ -138,13 +137,13 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 	 **/
 	signals [MINER_RESUMED] =
 		g_signal_new ("miner-resumed",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (TrackerMinerManagerClass, miner_resumed),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1,
-			      G_TYPE_STRING);
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (TrackerMinerManagerClass, miner_resumed),
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__STRING,
+		              G_TYPE_NONE, 1,
+		              G_TYPE_STRING);
 	/**
 	 * TrackerMinerManager::miner-activated
 	 * @manager: the #TrackerMinerManager
@@ -156,13 +155,13 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 	 **/
 	signals [MINER_ACTIVATED] =
 		g_signal_new ("miner-activated",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (TrackerMinerManagerClass, miner_activated),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1,
-			      G_TYPE_STRING);
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (TrackerMinerManagerClass, miner_activated),
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__STRING,
+		              G_TYPE_NONE, 1,
+		              G_TYPE_STRING);
 	/**
 	 * TrackerMinerManager::miner-deactivated
 	 * @manager: the #TrackerMinerManager
@@ -174,20 +173,21 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 	 **/
 	signals [MINER_DEACTIVATED] =
 		g_signal_new ("miner-deactivated",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (TrackerMinerManagerClass, miner_deactivated),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1,
-			      G_TYPE_STRING);
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (TrackerMinerManagerClass, miner_deactivated),
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__STRING,
+		              G_TYPE_NONE, 1,
+		              G_TYPE_STRING);
 
 	g_type_class_add_private (object_class, sizeof (TrackerMinerManagerPrivate));
 }
 
 static DBusGProxy *
 find_miner_proxy (TrackerMinerManager *manager,
-		  const gchar         *name)
+                  const gchar         *name,
+                  gboolean             try_suffix)
 {
 	TrackerMinerManagerPrivate *priv;
 	GHashTableIter iter;
@@ -200,6 +200,12 @@ find_miner_proxy (TrackerMinerManager *manager,
 		if (g_strcmp0 (name, (gchar *) value) == 0) {
 			return key;
 		}
+
+		if (try_suffix) {
+			if (g_str_has_suffix (value, name)) {
+				return key;
+			}
+		}
 	}
 
 	return NULL;
@@ -207,17 +213,17 @@ find_miner_proxy (TrackerMinerManager *manager,
 
 static void
 name_owner_changed_cb (DBusGProxy  *proxy,
-		       const gchar *name,
-		       const gchar *old_owner,
-		       const gchar *new_owner,
-		       gpointer     user_data)
+                       const gchar *name,
+                       const gchar *old_owner,
+                       const gchar *new_owner,
+                       gpointer     user_data)
 {
 	TrackerMinerManager *manager = user_data;
 	TrackerMinerManagerPrivate *priv;
 
 	priv = TRACKER_MINER_MANAGER_GET_PRIVATE (manager);
 
-	if (find_miner_proxy (manager, name) != NULL) {
+	if (find_miner_proxy (manager, name, FALSE) != NULL) {
 		if (new_owner && (!old_owner || !*old_owner)) {
 			g_signal_emit (manager, signals[MINER_ACTIVATED], 0, name);
 		} else if (old_owner && (!new_owner || !*new_owner)) {
@@ -228,9 +234,9 @@ name_owner_changed_cb (DBusGProxy  *proxy,
 
 static void
 miner_progress_changed (DBusGProxy  *proxy,
-			const gchar *status,
-			gdouble      progress,
-			gpointer     user_data)
+                        const gchar *status,
+                        gdouble      progress,
+                        gpointer     user_data)
 {
 	TrackerMinerManager *manager = user_data;
 	TrackerMinerManagerPrivate *priv;
@@ -245,7 +251,7 @@ miner_progress_changed (DBusGProxy  *proxy,
 
 static void
 miner_paused (DBusGProxy *proxy,
-	      gpointer    user_data)
+              gpointer    user_data)
 {
 	TrackerMinerManager *manager = user_data;
 	TrackerMinerManagerPrivate *priv;
@@ -260,7 +266,7 @@ miner_paused (DBusGProxy *proxy,
 
 static void
 miner_resumed (DBusGProxy *proxy,
-	       gpointer    user_data)
+               gpointer    user_data)
 {
 	TrackerMinerManager *manager = user_data;
 	TrackerMinerManagerPrivate *priv;
@@ -286,40 +292,40 @@ tracker_miner_manager_init (TrackerMinerManager *manager)
 
 	if (!priv->connection) {
 		g_critical ("Could not connect to the D-Bus session bus, %s",
-			    error ? error->message : "no error given.");
+		            error ? error->message : "no error given.");
 		g_clear_error (&error);
 	}
 
 	priv->proxy = dbus_g_proxy_new_for_name (priv->connection,
-						 DBUS_SERVICE_DBUS,
-						 DBUS_PATH_DBUS,
-						 DBUS_INTERFACE_DBUS);
+	                                         DBUS_SERVICE_DBUS,
+	                                         DBUS_PATH_DBUS,
+	                                         DBUS_INTERFACE_DBUS);
 
 	if (!priv->proxy) {
 		g_critical ("Could not get proxy for D-Bus service");
 	}
 
 	priv->miner_proxies = g_hash_table_new_full (NULL, NULL,
-						     (GDestroyNotify) g_object_unref,
-						     (GDestroyNotify) g_free);
+	                                             (GDestroyNotify) g_object_unref,
+	                                             (GDestroyNotify) g_free);
 
 	dbus_g_object_register_marshaller (tracker_marshal_VOID__STRING_DOUBLE,
-					   G_TYPE_NONE,
-					   G_TYPE_STRING,
-					   G_TYPE_DOUBLE,
-					   G_TYPE_INVALID);
+	                                   G_TYPE_NONE,
+	                                   G_TYPE_STRING,
+	                                   G_TYPE_DOUBLE,
+	                                   G_TYPE_INVALID);
 
 	dbus_g_proxy_add_signal (priv->proxy,
-				 "NameOwnerChanged",
-				 G_TYPE_STRING,
-				 G_TYPE_STRING,
-				 G_TYPE_STRING,
-				 G_TYPE_INVALID);
+	                         "NameOwnerChanged",
+	                         G_TYPE_STRING,
+	                         G_TYPE_STRING,
+	                         G_TYPE_STRING,
+	                         G_TYPE_INVALID);
 
 	dbus_g_proxy_connect_signal (priv->proxy,
-				     "NameOwnerChanged",
-				     G_CALLBACK (name_owner_changed_cb),
-				     manager, NULL);
+	                             "NameOwnerChanged",
+	                             G_CALLBACK (name_owner_changed_cb),
+	                             manager, NULL);
 
 	initialize_miners_data (manager);
 
@@ -330,30 +336,30 @@ tracker_miner_manager_init (TrackerMinerManager *manager)
 		data = m->data;
 
 		proxy = dbus_g_proxy_new_for_name (priv->connection,
-						   data->dbus_name,
-						   data->dbus_path,
-						   TRACKER_MINER_DBUS_INTERFACE);
+		                                   data->dbus_name,
+		                                   data->dbus_path,
+		                                   TRACKER_MINER_DBUS_INTERFACE);
 
 		dbus_g_proxy_add_signal (proxy,
-					 "Progress",
-					 G_TYPE_STRING,
-					 G_TYPE_DOUBLE,
-					 G_TYPE_INVALID);
+		                         "Progress",
+		                         G_TYPE_STRING,
+		                         G_TYPE_DOUBLE,
+		                         G_TYPE_INVALID);
 		dbus_g_proxy_add_signal (proxy, "Paused", G_TYPE_INVALID);
 		dbus_g_proxy_add_signal (proxy, "Resumed", G_TYPE_INVALID);
 
 		dbus_g_proxy_connect_signal (proxy,
-					     "Progress",
-					     G_CALLBACK (miner_progress_changed),
-					     manager, NULL);
+		                             "Progress",
+		                             G_CALLBACK (miner_progress_changed),
+		                             manager, NULL);
 		dbus_g_proxy_connect_signal (proxy,
-					     "Paused",
-					     G_CALLBACK (miner_paused),
-					     manager, NULL);
+		                             "Paused",
+		                             G_CALLBACK (miner_paused),
+		                             manager, NULL);
 		dbus_g_proxy_connect_signal (proxy,
-					     "Resumed",
-					     G_CALLBACK (miner_resumed),
-					     manager, NULL);
+		                             "Resumed",
+		                             G_CALLBACK (miner_resumed),
+		                             manager, NULL);
 
 		g_hash_table_insert (priv->miner_proxies, proxy, g_strdup (data->dbus_name));
 	}
@@ -427,20 +433,29 @@ tracker_miner_manager_get_running (TrackerMinerManager *manager)
 	}
 
 	if (!dbus_g_proxy_call (priv->proxy, "ListNames", &error,
-				G_TYPE_INVALID,
-				G_TYPE_STRV, &result,
-				G_TYPE_INVALID)) {
+	                        G_TYPE_INVALID,
+	                        G_TYPE_STRV, &result,
+	                        G_TYPE_INVALID)) {
 		g_critical ("Could not get a list of names registered on the session bus, %s",
-			    error ? error->message : "no error given");
+		            error ? error->message : "no error given");
 		g_clear_error (&error);
 		return NULL;
 	}
 
 	if (result) {
 		for (p = result; *p; p++) {
-			if (g_str_has_prefix (*p, TRACKER_MINER_DBUS_NAME_PREFIX)) {
-				list = g_slist_prepend (list, g_strdup (*p));
+			if (!g_str_has_prefix (*p, TRACKER_MINER_DBUS_NAME_PREFIX)) {
+				continue;
 			}
+
+			/* Special case miner-fs which has
+			 * additional D-Bus interface.
+			 */
+			if (strcmp (*p, "org.freedesktop.Tracker1.Miner.Files.Reindex") == 0) {
+				continue;
+			}
+
+			list = g_slist_prepend (list, g_strdup (*p));
 		}
 
 		list = g_slist_reverse (list);
@@ -453,8 +468,8 @@ tracker_miner_manager_get_running (TrackerMinerManager *manager)
 
 static gboolean
 crawler_check_file_cb (TrackerCrawler *crawler,
-		       GFile          *file,
-		       gpointer        user_data)
+                       GFile          *file,
+                       gpointer        user_data)
 {
 	TrackerMinerManager *manager;
 	TrackerMinerManagerPrivate *priv;
@@ -468,7 +483,7 @@ crawler_check_file_cb (TrackerCrawler *crawler,
 	priv = TRACKER_MINER_MANAGER_GET_PRIVATE (manager);
 
 	if (!g_str_has_suffix (path, ".desktop")) {
-	    return FALSE;
+		return FALSE;
 	}
 
 	key_file = g_key_file_new ();
@@ -513,13 +528,13 @@ crawler_check_file_cb (TrackerCrawler *crawler,
 
 static void
 crawler_finished_cb (TrackerCrawler *crawler,
-		     GQueue         *found,
-		     gboolean        was_interrupted,
-		     guint           directories_found,
-		     guint           directories_ignored,
-		     guint           files_found,
-		     guint           files_ignored,
-		     gpointer        user_data)
+                     GQueue         *found,
+                     gboolean        was_interrupted,
+                     guint           directories_found,
+                     guint           directories_ignored,
+                     guint           files_found,
+                     guint           files_ignored,
+                     gpointer        user_data)
 {
 	g_main_loop_quit (user_data);
 }
@@ -535,11 +550,11 @@ initialize_miners_data (TrackerMinerManager *manager)
 	main_loop = g_main_loop_new (NULL, FALSE);
 
 	g_signal_connect (crawler, "check-file",
-			  G_CALLBACK (crawler_check_file_cb),
-			  manager);
+	                  G_CALLBACK (crawler_check_file_cb),
+	                  manager);
 	g_signal_connect (crawler, "finished",
-			  G_CALLBACK (crawler_finished_cb),
-			  main_loop);
+	                  G_CALLBACK (crawler_finished_cb),
+	                  main_loop);
 
 	/* Go through service files */
 	file = g_file_new_for_path (TRACKER_MINERS_DIR);
@@ -593,9 +608,9 @@ tracker_miner_manager_get_available (TrackerMinerManager *manager)
  **/
 gboolean
 tracker_miner_manager_pause (TrackerMinerManager *manager,
-			     const gchar         *miner,
-			     const gchar         *reason,
-			     guint32             *cookie)
+                             const gchar         *miner,
+                             const gchar         *reason,
+                             guint32             *cookie)
 {
 	DBusGProxy *proxy;
 	const gchar *app_name;
@@ -605,7 +620,7 @@ tracker_miner_manager_pause (TrackerMinerManager *manager,
 	g_return_val_if_fail (miner != NULL, FALSE);
 	g_return_val_if_fail (reason != NULL, FALSE);
 
-	proxy = find_miner_proxy (manager, miner);
+	proxy = find_miner_proxy (manager, miner, TRUE);
 
 	if (!proxy) {
 		g_critical ("No D-Bus proxy found for miner '%s'", miner);
@@ -647,15 +662,15 @@ tracker_miner_manager_pause (TrackerMinerManager *manager,
  **/
 gboolean
 tracker_miner_manager_resume (TrackerMinerManager *manager,
-			      const gchar         *miner,
-			      guint32              cookie)
+                              const gchar         *miner,
+                              guint32              cookie)
 {
 	DBusGProxy *proxy;
 	GError *error = NULL;
 
 	g_return_val_if_fail (TRACKER_IS_MINER_MANAGER (manager), FALSE);
 	g_return_val_if_fail (miner != NULL, FALSE);
-	proxy = find_miner_proxy (manager, miner);
+	proxy = find_miner_proxy (manager, miner, TRUE);
 
 	if (!proxy) {
 		g_critical ("No D-Bus proxy found for miner '%s'", miner);
@@ -684,7 +699,7 @@ tracker_miner_manager_resume (TrackerMinerManager *manager,
  **/
 gboolean
 tracker_miner_manager_is_active (TrackerMinerManager *manager,
-				 const gchar         *miner)
+                                 const gchar         *miner)
 {
 	TrackerMinerManagerPrivate *priv;
 	GError *error = NULL;
@@ -696,12 +711,12 @@ tracker_miner_manager_is_active (TrackerMinerManager *manager,
 	priv = TRACKER_MINER_MANAGER_GET_PRIVATE (manager);
 
 	if (!dbus_g_proxy_call (priv->proxy, "NameHasOwner", &error,
-				G_TYPE_STRING, miner,
-				G_TYPE_INVALID,
-				G_TYPE_BOOLEAN, &active,
-				G_TYPE_INVALID)) {
+	                        G_TYPE_STRING, miner,
+	                        G_TYPE_INVALID,
+	                        G_TYPE_BOOLEAN, &active,
+	                        G_TYPE_INVALID)) {
 		g_critical ("Could not check whether miner '%s' is currently active: %s",
-			    miner, error ? error->message : "no error given");
+		            miner, error ? error->message : "no error given");
 		g_error_free (error);
 		return FALSE;
 	}
@@ -722,9 +737,9 @@ tracker_miner_manager_is_active (TrackerMinerManager *manager,
  **/
 gboolean
 tracker_miner_manager_get_status (TrackerMinerManager  *manager,
-				  const gchar          *miner,
-				  gchar               **status,
-				  gdouble              *progress)
+                                  const gchar          *miner,
+                                  gchar               **status,
+                                  gdouble              *progress)
 {
 	DBusGProxy *proxy;
 	GError *error = NULL;
@@ -734,7 +749,7 @@ tracker_miner_manager_get_status (TrackerMinerManager  *manager,
 	g_return_val_if_fail (TRACKER_IS_MINER_MANAGER (manager), FALSE);
 	g_return_val_if_fail (miner != NULL, FALSE);
 
-	proxy = find_miner_proxy (manager, miner);
+	proxy = find_miner_proxy (manager, miner, TRUE);
 
 	if (!proxy) {
 		g_critical ("No D-Bus proxy found for miner '%s'", miner);
@@ -745,11 +760,11 @@ tracker_miner_manager_get_status (TrackerMinerManager  *manager,
 
 	if (error) {
 		/* We handle this error as a special case, some
-		 * plugins don't have .service files. 
+		 * plugins don't have .service files.
 		 */
 		if (error->code != DBUS_GERROR_SERVICE_UNKNOWN) {
 			g_critical ("Could not get miner progress for '%s': %s", miner,
-				    error->message);
+			            error->message);
 		}
 
 		g_error_free (error);
@@ -761,7 +776,7 @@ tracker_miner_manager_get_status (TrackerMinerManager  *manager,
 
 	if (error) {
 		g_critical ("Could not get miner status for '%s': %s", miner,
-			    error->message);
+		            error->message);
 		g_error_free (error);
 
 		return FALSE;
@@ -797,9 +812,9 @@ tracker_miner_manager_get_status (TrackerMinerManager  *manager,
  **/
 gboolean
 tracker_miner_manager_is_paused (TrackerMinerManager *manager,
-				 const gchar         *miner,
-				 GStrv               *applications,
-				 GStrv               *reasons)
+                                 const gchar         *miner,
+                                 GStrv               *applications,
+                                 GStrv               *reasons)
 {
 	DBusGProxy *proxy;
 	GStrv apps, r;
@@ -817,7 +832,7 @@ tracker_miner_manager_is_paused (TrackerMinerManager *manager,
 	g_return_val_if_fail (TRACKER_IS_MINER_MANAGER (manager), TRUE);
 	g_return_val_if_fail (miner != NULL, TRUE);
 
-	proxy = find_miner_proxy (manager, miner);
+	proxy = find_miner_proxy (manager, miner, TRUE);
 
 	if (!proxy) {
 		g_critical ("No D-Bus proxy found for miner '%s'", miner);
@@ -828,7 +843,7 @@ tracker_miner_manager_is_paused (TrackerMinerManager *manager,
 
 	if (error) {
 		g_critical ("Could not get pause details for miner '%s': %s", miner,
-			    error->message);
+		            error->message);
 		g_error_free (error);
 
 		return TRUE;
@@ -862,7 +877,7 @@ tracker_miner_manager_is_paused (TrackerMinerManager *manager,
  **/
 const gchar *
 tracker_miner_manager_get_display_name (TrackerMinerManager *manager,
-					const gchar         *miner)
+                                        const gchar         *miner)
 {
 	TrackerMinerManagerPrivate *priv;
 	GList *m;
@@ -894,7 +909,7 @@ tracker_miner_manager_get_display_name (TrackerMinerManager *manager,
  **/
 const gchar *
 tracker_miner_manager_get_description (TrackerMinerManager *manager,
-				       const gchar         *miner)
+                                       const gchar         *miner)
 {
 	TrackerMinerManagerPrivate *priv;
 	GList *m;
@@ -914,3 +929,55 @@ tracker_miner_manager_get_description (TrackerMinerManager *manager,
 
 	return NULL;
 }
+
+
+/**
+ * tracker_miner_manager_writeback:
+ * @manager: a #TrackerMinerManager.
+ * @miner: miner reference
+ * @subjects: subjects to mark as writeback
+ *
+ * Asks @miner to mark @subjects as writeback
+ *
+ * Returns: %TRUE if the miner was asked to writeback successfully.
+ **/
+gboolean
+tracker_miner_manager_writeback (TrackerMinerManager *manager,
+                                 const gchar         *miner,
+                                 const gchar        **subjects)
+{
+	DBusGProxy *proxy;
+	const gchar *app_name;
+	GError *error = NULL;
+
+	g_return_val_if_fail (TRACKER_IS_MINER_MANAGER (manager), FALSE);
+	g_return_val_if_fail (miner != NULL, FALSE);
+
+	proxy = find_miner_proxy (manager, miner, TRUE);
+
+	if (!proxy) {
+		g_warning ("No D-Bus proxy found for miner '%s'", miner);
+		return FALSE;
+	}
+
+	/* Find a reasonable app name */
+	app_name = g_get_application_name ();
+
+	if (!app_name) {
+		app_name = g_get_prgname ();
+	}
+
+	if (!app_name) {
+		app_name = "TrackerMinerManager client";
+	}
+
+	org_freedesktop_Tracker1_Miner_writeback (proxy, subjects, &error);
+
+	if (error) {
+		g_warning ("Could not writeback miner '%s': %s", miner, error->message);
+		g_error_free (error);
+		return FALSE;
+	}
+
+	return TRUE;
+}
