@@ -117,6 +117,8 @@ typedef struct {
 	/* Tags and data */
 	GstTagList     *tagcache;
 
+	gboolean        is_content_encrypted;
+
 	unsigned char  *album_art_data;
 	guint           album_art_size;
 	const gchar    *album_art_mime;
@@ -124,12 +126,16 @@ typedef struct {
 } MetadataExtractor;
 
 static void extract_gstreamer_audio (const gchar          *uri,
+                                     TrackerSparqlBuilder *preupdate,
                                      TrackerSparqlBuilder *metadata);
 static void extract_gstreamer_video (const gchar          *uri,
+                                     TrackerSparqlBuilder *preupdate,
                                      TrackerSparqlBuilder *metadata);
 static void extract_gstreamer_image (const gchar          *uri,
+                                     TrackerSparqlBuilder *preupdate,
                                      TrackerSparqlBuilder *metadata);
 static void extract_gstreamer_3gpp  (const gchar          *uri,
+                                     TrackerSparqlBuilder *preupdate,
                                      TrackerSparqlBuilder *metadata);
 
 static TrackerExtractData data[] = {
@@ -281,9 +287,9 @@ add_y_date_gst_tag (TrackerSparqlBuilder  *metadata,
 	ret = gst_tag_list_get_date (tag_list, tag, &date);
 
 	if (ret) {
-		gchar buf[10];
+		gchar buf[25];
 
-		if (g_date_strftime (buf, 10, "%Y", date)) {
+		if (g_date_strftime (buf, 25, "%Y-%m-%dT%H:%M:%S%z", date)) {
 			tracker_sparql_builder_predicate (metadata, key);
 			tracker_sparql_builder_object_unvalidated (metadata, buf);
 		}
@@ -445,6 +451,7 @@ extract_stream_metadata_decodebin (MetadataExtractor *extractor,
 static void
 extract_metadata (MetadataExtractor      *extractor,
                   const gchar            *uri,
+                  TrackerSparqlBuilder   *preupdate,
                   TrackerSparqlBuilder   *metadata,
                   gchar                 **album,
                   gchar                 **scount)
@@ -452,12 +459,14 @@ extract_metadata (MetadataExtractor      *extractor,
 	gchar *s;
 	gboolean ret;
 	gint count;
+	gboolean needs_audio = FALSE;
 
 	g_return_if_fail (extractor != NULL);
 	g_return_if_fail (metadata != NULL);
 
 	if (extractor->tagcache) {
 		gchar *performer_uri = NULL;
+		gchar *composer_uri = NULL;
 		gchar *album_uri = NULL;
 
 		if (extractor->mime == EXTRACT_MIME_3GPP) {
@@ -489,11 +498,13 @@ extract_metadata (MetadataExtractor      *extractor,
 			if (s) {
 				performer_uri = tracker_uri_printf_escaped ("urn:artist:%s", s);
 
-				tracker_sparql_builder_subject_iri (metadata, performer_uri);
-				tracker_sparql_builder_predicate (metadata, "a");
-				tracker_sparql_builder_object (metadata, "nmm:Artist");
-				tracker_sparql_builder_predicate (metadata, "nmm:artistName");
-				tracker_sparql_builder_object_unvalidated (metadata, s);
+				tracker_sparql_builder_insert_open (preupdate, NULL);
+				tracker_sparql_builder_subject_iri (preupdate, performer_uri);
+				tracker_sparql_builder_predicate (preupdate, "a");
+				tracker_sparql_builder_object (preupdate, "nmm:Artist");
+				tracker_sparql_builder_predicate (preupdate, "nmm:artistName");
+				tracker_sparql_builder_object_unvalidated (preupdate, s);
+				tracker_sparql_builder_insert_close (preupdate);
 
 				g_free (s);
 			}
@@ -502,54 +513,113 @@ extract_metadata (MetadataExtractor      *extractor,
 			gst_tag_list_get_string (extractor->tagcache, GST_TAG_COMPOSER, &s);
 
 			if (s) {
-				gchar *canonical_uri = tracker_uri_printf_escaped ("urn:artist:%s", s);
+				composer_uri = tracker_uri_printf_escaped ("urn:artist:%s", s);
 
-				tracker_sparql_builder_subject_iri (metadata, canonical_uri);
-				tracker_sparql_builder_predicate (metadata, "a");
-				tracker_sparql_builder_object (metadata, "nmm:Artist");
+				tracker_sparql_builder_insert_open (preupdate, NULL);
 
-				/*
-				if (extractor->mime == EXTRACT_MIME_AUDIO) {
-					tracker_sparql_builder_object (metadata, "nmm:composer");
-				} else {
-					tracker_sparql_builder_object (metadata, "nmm:director");
-				}
-				*/
-				tracker_sparql_builder_predicate (metadata, "nmm:artistName");
-				tracker_sparql_builder_object_unvalidated (metadata, s);
+				tracker_sparql_builder_subject_iri (preupdate, composer_uri);
+				tracker_sparql_builder_predicate (preupdate, "a");
+				tracker_sparql_builder_object (preupdate, "nmm:Artist");
+
+				tracker_sparql_builder_predicate (preupdate, "nmm:artistName");
+				tracker_sparql_builder_object_unvalidated (preupdate, s);
+
+				tracker_sparql_builder_insert_close (preupdate);
 
-				g_free (canonical_uri);
 				g_free (s);
 			}
 
 		}
 
-		if (extractor->mime == EXTRACT_MIME_AUDIO) {
-			/* Audio */
-			s = NULL;
-			gst_tag_list_get_string (extractor->tagcache, GST_TAG_ALBUM, &s);
-			if (s) {
-				album_uri = tracker_uri_printf_escaped ("urn:album:%s", s);
-				tracker_sparql_builder_subject_iri (metadata, album_uri);
-				tracker_sparql_builder_predicate (metadata, "a");
-				tracker_sparql_builder_object (metadata, "nmm:MusicAlbum");
-				tracker_sparql_builder_predicate (metadata, "nmm:albumTitle");
-				tracker_sparql_builder_object_unvalidated (metadata, s);
-				add_uint_gst_tag (metadata, album_uri, "nmm:albumTrackCount", extractor->tagcache, GST_TAG_TRACK_COUNT);
-				add_uint_gst_tag (metadata, album_uri, "nmm:setNumber", extractor->tagcache, GST_TAG_ALBUM_VOLUME_NUMBER);
-				*album = s;
+		/* Audio */
+		s = NULL;
+		gst_tag_list_get_string (extractor->tagcache, GST_TAG_ALBUM, &s);
+		if (s) {
+			gboolean has_it;
+			guint count;
+
+			needs_audio = TRUE;
+
+			album_uri = tracker_uri_printf_escaped ("urn:album:%s", s);
+
+			tracker_sparql_builder_insert_open (preupdate, NULL);
+
+			tracker_sparql_builder_subject_iri (preupdate, album_uri);
+			tracker_sparql_builder_predicate (preupdate, "a");
+			tracker_sparql_builder_object (preupdate, "nmm:MusicAlbum");
+			tracker_sparql_builder_predicate (preupdate, "nmm:albumTitle");
+			tracker_sparql_builder_object_unvalidated (preupdate, s);
+
+			tracker_sparql_builder_insert_close (preupdate);
+
+			has_it = gst_tag_list_get_uint (extractor->tagcache, 
+			                                GST_TAG_TRACK_COUNT, 
+			                                &count);
+
+			if (has_it) {
+				tracker_sparql_builder_delete_open (preupdate, NULL);
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:albumTrackCount");
+				tracker_sparql_builder_object_variable (preupdate, "unknown");
+				tracker_sparql_builder_delete_close (preupdate);
+
+				tracker_sparql_builder_where_open (preupdate);
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:albumTrackCount");
+				tracker_sparql_builder_object_variable (preupdate, "unknown");
+				tracker_sparql_builder_where_close (preupdate);
+
+				tracker_sparql_builder_insert_open (preupdate, NULL);
+
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:albumTrackCount");
+				tracker_sparql_builder_object_int64 (preupdate, count);
+
+				tracker_sparql_builder_insert_close (preupdate);
 			}
+
+			has_it = gst_tag_list_get_uint (extractor->tagcache, 
+			                                GST_TAG_ALBUM_VOLUME_NUMBER, 
+			                                &count);
+
+			if (has_it) {
+				tracker_sparql_builder_delete_open (preupdate, NULL);
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:setNumber");
+				tracker_sparql_builder_object_variable (preupdate, "unknown");
+				tracker_sparql_builder_delete_close (preupdate);
+
+				tracker_sparql_builder_where_open (preupdate);
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:setNumber");
+				tracker_sparql_builder_object_variable (preupdate, "unknown");
+				tracker_sparql_builder_where_close (preupdate);
+
+				tracker_sparql_builder_insert_open (preupdate, NULL);
+
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:setNumber");
+				tracker_sparql_builder_object_int64 (preupdate, count);
+
+				tracker_sparql_builder_insert_close (preupdate);
+			}
+
+			*album = s;
 		}
 
-		tracker_sparql_builder_subject_iri (metadata, uri);
+		if (extractor->mime == EXTRACT_MIME_AUDIO)
+			needs_audio = TRUE;
+
 		tracker_sparql_builder_predicate (metadata, "a");
 
-		if (extractor->mime == EXTRACT_MIME_AUDIO) {
+		if (needs_audio) {
 			tracker_sparql_builder_object (metadata, "nmm:MusicPiece");
 			tracker_sparql_builder_object (metadata, "nfo:Audio");
-		} else if (extractor->mime == EXTRACT_MIME_VIDEO) {
+		} 
+
+		if (extractor->mime == EXTRACT_MIME_VIDEO) {
 			tracker_sparql_builder_object (metadata, "nmm:Video");
-		} else {
+		} else if (!needs_audio) {
 			tracker_sparql_builder_object (metadata, "nfo:Image");
 		}
 
@@ -568,17 +638,26 @@ extract_metadata (MetadataExtractor      *extractor,
 		add_y_date_gst_tag (metadata, uri, "nie:contentCreated", extractor->tagcache, GST_TAG_DATE);
 		add_string_gst_tag (metadata, uri, "nie:comment", extractor->tagcache, GST_TAG_COMMENT);
 
+		if (extractor->is_content_encrypted) {
+			tracker_sparql_builder_predicate (metadata, "nfo:isContentEncrypted");
+			tracker_sparql_builder_object_boolean (metadata, TRUE);
+		}
+
 		if (extractor->mime == EXTRACT_MIME_VIDEO) {
 			add_string_gst_tag (metadata, uri, "dc:source", extractor->tagcache, GST_TAG_CLASSIFICATION);
 
 			if (performer_uri) {
 				tracker_sparql_builder_predicate (metadata, "nmm:leadActor");
 				tracker_sparql_builder_object_iri (metadata, performer_uri);
-				g_free (performer_uri);
+			}
+
+			if (composer_uri) {
+				tracker_sparql_builder_predicate (metadata, "nmm:director");
+				tracker_sparql_builder_object_iri (metadata, composer_uri);
 			}
 		}
 
-		if (extractor->mime == EXTRACT_MIME_AUDIO) {
+		if (needs_audio) {
 			/* Audio */
 			ret = gst_tag_list_get_uint (extractor->tagcache, GST_TAG_TRACK_COUNT, &count);
 			if (ret) {
@@ -595,16 +674,23 @@ extract_metadata (MetadataExtractor      *extractor,
 			if (performer_uri) {
 				tracker_sparql_builder_predicate (metadata, "nmm:performer");
 				tracker_sparql_builder_object_iri (metadata, performer_uri);
-				g_free (performer_uri);
+			}
+
+			if (composer_uri) {
+				tracker_sparql_builder_predicate (metadata, "nmm:composer");
+				tracker_sparql_builder_object_iri (metadata, composer_uri);
 			}
 
 			if (album_uri) {
 				tracker_sparql_builder_predicate (metadata, "nmm:musicAlbum");
 				tracker_sparql_builder_object_iri (metadata, album_uri);
-				g_free (album_uri);
 			}
 		}
 
+		g_free (performer_uri);
+		g_free (composer_uri);
+		g_free (album_uri);
+
 		add_string_gst_tag (metadata, uri, "nfo:codec", extractor->tagcache, GST_TAG_AUDIO_CODEC);
 	}
 
@@ -789,6 +875,22 @@ poll_for_ready (MetadataExtractor *extractor,
 			gchar  *error_message;
 
 			gst_message_parse_error (message, &lerror, &error_message);
+
+#if (GST_VERSION_MICRO >= 20)
+			if (lerror->domain == GST_STREAM_ERROR) {
+				if (lerror->code == GST_STREAM_ERROR_DECRYPT ||
+				    lerror->code == GST_STREAM_ERROR_DECRYPT_NOKEY) {
+					/* also extract metadata from encrypted streams */
+
+					extractor->is_content_encrypted = TRUE;
+
+					g_free (error_message);
+					g_error_free (lerror);
+					break;
+				}
+			}
+#endif
+
 			gst_message_unref (message);
 			g_warning ("Got error :%s", error_message);
 			g_free (error_message);
@@ -893,6 +995,7 @@ create_tagreadbin_pipeline (MetadataExtractor *extractor, const gchar *uri)
 
 static void
 tracker_extract_gstreamer (const gchar *uri,
+                           TrackerSparqlBuilder  *preupdate,
                            TrackerSparqlBuilder  *metadata,
                            ExtractMime  type)
 {
@@ -963,7 +1066,7 @@ tracker_extract_gstreamer (const gchar *uri,
 	album = NULL;
 	scount = NULL;
 
-	extract_metadata (extractor, uri, metadata, &album, &scount);
+	extract_metadata (extractor, uri, preupdate, metadata, &album, &scount);
 
 	tracker_albumart_process (extractor->album_art_data,
 	                          extractor->album_art_size,
@@ -989,27 +1092,35 @@ tracker_extract_gstreamer (const gchar *uri,
 
 
 static void
-extract_gstreamer_audio (const gchar *uri, TrackerSparqlBuilder *metadata)
+extract_gstreamer_audio (const gchar          *uri,
+                         TrackerSparqlBuilder *preupdate,
+			 TrackerSparqlBuilder *metadata)
 {
-	tracker_extract_gstreamer (uri, metadata, EXTRACT_MIME_AUDIO);
+	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_AUDIO);
 }
 
 static void
-extract_gstreamer_video (const gchar *uri, TrackerSparqlBuilder *metadata)
+extract_gstreamer_video (const gchar          *uri,
+                         TrackerSparqlBuilder *preupdate,
+			 TrackerSparqlBuilder *metadata)
 {
-	tracker_extract_gstreamer (uri, metadata, EXTRACT_MIME_VIDEO);
+	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_VIDEO);
 }
 
 static void
-extract_gstreamer_image (const gchar *uri, TrackerSparqlBuilder *metadata)
+extract_gstreamer_image (const gchar          *uri,
+                         TrackerSparqlBuilder *preupdate,
+			 TrackerSparqlBuilder *metadata)
 {
-	tracker_extract_gstreamer (uri, metadata, EXTRACT_MIME_IMAGE);
+	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_IMAGE);
 }
 
 static void
-extract_gstreamer_3gpp (const gchar *uri, TrackerSparqlBuilder *metadata)
+extract_gstreamer_3gpp (const gchar          *uri,
+                        TrackerSparqlBuilder *preupdate,
+			TrackerSparqlBuilder *metadata)
 {
-	tracker_extract_gstreamer (uri, metadata, EXTRACT_MIME_3GPP);
+	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_3GPP);
 }
 
 TrackerExtractData *
