@@ -124,30 +124,12 @@ static gpointer _gtk_tree_path_copy0 (gpointer self) {
 GtkTreePath* tracker_result_grid_GetSelectedPath (TrackerResultGrid* self) {
 #line 126 "tracker-result-grid.c"
 	GtkTreePath* result;
-	GList* l;
-	gboolean _tmp0_;
 #line 61 "tracker-result-grid.gs"
 	g_return_val_if_fail (self != NULL, NULL);
+#line 62 "tracker-result-grid.gs"
+	if (gtk_icon_view_get_selected_items (self->iconview) != NULL) {
 #line 132 "tracker-result-grid.c"
-	l = NULL;
-#line 64 "tracker-result-grid.gs"
-	l = gtk_icon_view_get_selected_items (self->iconview);
-#line 136 "tracker-result-grid.c"
-	_tmp0_ = FALSE;
-#line 66 "tracker-result-grid.gs"
-	if (l != NULL) {
-#line 66 "tracker-result-grid.gs"
-		_tmp0_ = ((const GtkTreePath*) l->data) != NULL;
-#line 142 "tracker-result-grid.c"
-	} else {
-#line 66 "tracker-result-grid.gs"
-		_tmp0_ = FALSE;
-#line 146 "tracker-result-grid.c"
-	}
-#line 66 "tracker-result-grid.gs"
-	if (_tmp0_) {
-#line 150 "tracker-result-grid.c"
-		result = _gtk_tree_path_copy0 ((const GtkTreePath*) l->data);
+		result = _gtk_tree_path_copy0 ((const GtkTreePath*) gtk_icon_view_get_selected_items (self->iconview)->data);
 		return result;
 	}
 	result = NULL;
@@ -155,25 +137,25 @@ GtkTreePath* tracker_result_grid_GetSelectedPath (TrackerResultGrid* self) {
 }
 
 
-#line 72 "tracker-result-grid.gs"
+#line 68 "tracker-result-grid.gs"
 const char* tracker_result_grid_GetSelectedUri (TrackerResultGrid* self) {
-#line 161 "tracker-result-grid.c"
+#line 143 "tracker-result-grid.c"
 	const char* result;
 	GtkTreeIter iter = {0};
 	const char* uri;
 	GtkTreePath* path;
-#line 72 "tracker-result-grid.gs"
+#line 68 "tracker-result-grid.gs"
 	g_return_val_if_fail (self != NULL, NULL);
-#line 168 "tracker-result-grid.c"
+#line 150 "tracker-result-grid.c"
 	uri = NULL;
 	path = tracker_result_grid_GetSelectedPath (self);
-#line 77 "tracker-result-grid.gs"
+#line 73 "tracker-result-grid.gs"
 	if (path != NULL) {
-#line 78 "tracker-result-grid.gs"
+#line 74 "tracker-result-grid.gs"
 		gtk_tree_model_get_iter ((GtkTreeModel*) self->store, &iter, path);
-#line 79 "tracker-result-grid.gs"
+#line 75 "tracker-result-grid.gs"
 		gtk_tree_model_get ((GtkTreeModel*) self->store, &iter, RESULT_COLUMNS_Uri, &uri, -1);
-#line 177 "tracker-result-grid.c"
+#line 159 "tracker-result-grid.c"
 		result = uri;
 		_gtk_tree_path_free0 (path);
 		return result;
@@ -184,17 +166,17 @@ const char* tracker_result_grid_GetSelectedUri (TrackerResultGrid* self) {
 }
 
 
-#line 121 "tracker-result-grid.gs"
+#line 117 "tracker-result-grid.gs"
 void tracker_result_grid_RefreshQuery (TrackerResultGrid* self) {
-#line 190 "tracker-result-grid.c"
+#line 172 "tracker-result-grid.c"
 	GError * _inner_error_;
-#line 121 "tracker-result-grid.gs"
+#line 117 "tracker-result-grid.gs"
 	g_return_if_fail (self != NULL);
-#line 194 "tracker-result-grid.c"
+#line 176 "tracker-result-grid.c"
 	_inner_error_ = NULL;
-#line 122 "tracker-result-grid.gs"
+#line 118 "tracker-result-grid.gs"
 	if (self->priv->_query != NULL) {
-#line 198 "tracker-result-grid.c"
+#line 180 "tracker-result-grid.c"
 		char** _tmp2_;
 		gint results_length2;
 		gint results_length1;
@@ -205,11 +187,11 @@ void tracker_result_grid_RefreshQuery (TrackerResultGrid* self) {
 		GtkTreeIter iter = {0};
 		results = (_tmp2_ = tracker_query_Search (self->priv->_query, &_tmp0_, &_tmp1_), results_length1 = _tmp0_, results_length2 = _tmp1_, _tmp2_);
 		has_results = FALSE;
-#line 127 "tracker-result-grid.gs"
+#line 123 "tracker-result-grid.gs"
 		gtk_list_store_clear (self->store);
-#line 129 "tracker-result-grid.gs"
+#line 125 "tracker-result-grid.gs"
 		if (results == NULL) {
-#line 213 "tracker-result-grid.c"
+#line 195 "tracker-result-grid.c"
 			results = (_vala_array_free (results, results_length1 * results_length2, (GDestroyNotify) g_free), NULL);
 			return;
 		}
@@ -217,21 +199,21 @@ void tracker_result_grid_RefreshQuery (TrackerResultGrid* self) {
 			char** uri_collection;
 			int uri_collection_length1;
 			int uri_it;
-#line 131 "tracker-result-grid.gs"
+#line 127 "tracker-result-grid.gs"
 			uri_collection = results;
-#line 223 "tracker-result-grid.c"
+#line 205 "tracker-result-grid.c"
 			uri_collection_length1 = results_length1 * results_length2;
 			for (uri_it = 0; uri_it < (results_length1 * results_length2); uri_it = uri_it + 1) {
 				char* uri;
 				uri = g_strdup (uri_collection[uri_it]);
 				{
-#line 132 "tracker-result-grid.gs"
+#line 128 "tracker-result-grid.gs"
 					if (g_str_has_prefix (uri, "file://")) {
-#line 231 "tracker-result-grid.c"
+#line 213 "tracker-result-grid.c"
 						GFile* file;
-#line 134 "tracker-result-grid.gs"
+#line 130 "tracker-result-grid.gs"
 						has_results = TRUE;
-#line 235 "tracker-result-grid.c"
+#line 217 "tracker-result-grid.c"
 						file = g_file_new_for_uri (uri);
 						{
 							GFileInfo* info;
@@ -239,32 +221,32 @@ void tracker_result_grid_RefreshQuery (TrackerResultGrid* self) {
 							GdkPixbuf* _tmp3_;
 							info = g_file_query_info (file, "standard::display-name,standard::icon,thumbnail::path", G_FILE_QUERY_INFO_NONE, NULL, &_inner_error_);
 							if (_inner_error_ != NULL) {
-								goto __catch2_g_error;
-								goto __finally2;
+								goto __catch3_g_error;
+								goto __finally3;
 							}
 							filetype = g_file_info_get_file_type (info);
-#line 143 "tracker-result-grid.gs"
+#line 139 "tracker-result-grid.gs"
 							gtk_list_store_append (self->store, &iter);
-#line 144 "tracker-result-grid.gs"
+#line 140 "tracker-result-grid.gs"
 							gtk_list_store_set (self->store, &iter, RESULT_COLUMNS_Uri, uri, RESULT_COLUMNS_Icon, _tmp3_ = tracker_utils_GetThumbNail (info, 64, 48, gtk_widget_get_screen ((GtkWidget*) self)), RESULT_COLUMNS_DisplayName, g_file_info_get_display_name (info), RESULT_COLUMNS_IsDirectory, filetype == G_FILE_TYPE_DIRECTORY, -1, -1);
-#line 251 "tracker-result-grid.c"
+#line 233 "tracker-result-grid.c"
 							_g_object_unref0 (_tmp3_);
 							_g_object_unref0 (info);
 						}
-						goto __finally2;
-						__catch2_g_error:
+						goto __finally3;
+						__catch3_g_error:
 						{
 							GError * e;
 							e = _inner_error_;
 							_inner_error_ = NULL;
 							{
-#line 148 "tracker-result-grid.gs"
+#line 144 "tracker-result-grid.gs"
 								g_print ("Could not get file info for %s\n", uri);
-#line 264 "tracker-result-grid.c"
+#line 246 "tracker-result-grid.c"
 								_g_error_free0 (e);
 							}
 						}
-						__finally2:
+						__finally3:
 						if (_inner_error_ != NULL) {
 							_g_object_unref0 (file);
 							_g_free0 (uri);
@@ -279,16 +261,16 @@ void tracker_result_grid_RefreshQuery (TrackerResultGrid* self) {
 				}
 			}
 		}
-#line 151 "tracker-result-grid.gs"
+#line 147 "tracker-result-grid.gs"
 		if (has_results) {
-#line 285 "tracker-result-grid.c"
+#line 267 "tracker-result-grid.c"
 			GtkTreePath* path;
 			path = gtk_tree_path_new_from_string ("0:0:0");
-#line 153 "tracker-result-grid.gs"
+#line 149 "tracker-result-grid.gs"
 			if (path != NULL) {
-#line 154 "tracker-result-grid.gs"
+#line 150 "tracker-result-grid.gs"
 				gtk_icon_view_select_path (self->iconview, path);
-#line 292 "tracker-result-grid.c"
+#line 274 "tracker-result-grid.c"
 			}
 			_gtk_tree_path_free0 (path);
 		}
@@ -297,35 +279,35 @@ void tracker_result_grid_RefreshQuery (TrackerResultGrid* self) {
 }
 
 
-#line 159 "tracker-result-grid.gs"
+#line 155 "tracker-result-grid.gs"
 void tracker_result_grid_ActivateUri (TrackerResultGrid* self, const GtkTreePath* path) {
-#line 303 "tracker-result-grid.c"
+#line 285 "tracker-result-grid.c"
 	GtkTreeIter iter = {0};
 	gboolean is_dir;
 	const char* uri;
-#line 159 "tracker-result-grid.gs"
+#line 155 "tracker-result-grid.gs"
 	g_return_if_fail (self != NULL);
-#line 159 "tracker-result-grid.gs"
+#line 155 "tracker-result-grid.gs"
 	g_return_if_fail (path != NULL);
-#line 311 "tracker-result-grid.c"
+#line 293 "tracker-result-grid.c"
 	is_dir = FALSE;
-#line 163 "tracker-result-grid.gs"
+#line 159 "tracker-result-grid.gs"
 	gtk_tree_model_get_iter ((GtkTreeModel*) self->store, &iter, path);
-#line 315 "tracker-result-grid.c"
+#line 297 "tracker-result-grid.c"
 	uri = NULL;
-#line 165 "tracker-result-grid.gs"
+#line 161 "tracker-result-grid.gs"
 	gtk_tree_model_get ((GtkTreeModel*) self->store, &iter, RESULT_COLUMNS_Uri, &uri, -1);
-#line 166 "tracker-result-grid.gs"
+#line 162 "tracker-result-grid.gs"
 	gtk_tree_model_get ((GtkTreeModel*) self->store, &iter, RESULT_COLUMNS_IsDirectory, &is_dir, -1);
-#line 168 "tracker-result-grid.gs"
+#line 164 "tracker-result-grid.gs"
 	tracker_utils_OpenUri (uri, is_dir);
-#line 323 "tracker-result-grid.c"
+#line 305 "tracker-result-grid.c"
 }
 
 
 #line 43 "tracker-result-grid.gs"
 TrackerResultGrid* tracker_result_grid_construct (GType object_type) {
-#line 329 "tracker-result-grid.c"
+#line 311 "tracker-result-grid.c"
 	TrackerResultGrid * self;
 	self = g_object_newv (object_type, 0, NULL);
 	return self;
@@ -336,7 +318,7 @@ TrackerResultGrid* tracker_result_grid_construct (GType object_type) {
 TrackerResultGrid* tracker_result_grid_new (void) {
 #line 43 "tracker-result-grid.gs"
 	return tracker_result_grid_construct (TYPE_TRACKER_RESULT_GRID);
-#line 340 "tracker-result-grid.c"
+#line 322 "tracker-result-grid.c"
 }
 
 
@@ -356,7 +338,7 @@ static gpointer _g_object_ref0 (gpointer self) {
 static void _lambda2_ (TrackerResultGrid* self) {
 #line 57 "tracker-result-grid.gs"
 	tracker_result_grid_RefreshQuery (self);
-#line 360 "tracker-result-grid.c"
+#line 342 "tracker-result-grid.c"
 }
 
 
@@ -368,7 +350,7 @@ static void __lambda2__tracker_query_search_settings_changed (TrackerQuery* _sen
 static void _lambda3_ (TrackerResultGrid* self) {
 #line 59 "tracker-result-grid.gs"
 	gtk_list_store_clear (self->store);
-#line 372 "tracker-result-grid.c"
+#line 354 "tracker-result-grid.c"
 }
 
 
@@ -381,7 +363,7 @@ void tracker_result_grid_set_Query (TrackerResultGrid* self, TrackerQuery* value
 	g_return_if_fail (self != NULL);
 #line 54 "tracker-result-grid.gs"
 	if (value != NULL) {
-#line 385 "tracker-result-grid.c"
+#line 367 "tracker-result-grid.c"
 		TrackerQuery* _tmp0_;
 #line 55 "tracker-result-grid.gs"
 		self->priv->_query = (_tmp0_ = _g_object_ref0 (value), _g_object_unref0 (self->priv->_query), _tmp0_);
@@ -389,15 +371,15 @@ void tracker_result_grid_set_Query (TrackerResultGrid* self, TrackerQuery* value
 		g_signal_connect_object (self->priv->_query, "search-settings-changed", (GCallback) __lambda2__tracker_query_search_settings_changed, self, 0);
 #line 58 "tracker-result-grid.gs"
 		g_signal_connect_object (self->priv->_query, "clear-search-results", (GCallback) __lambda3__tracker_query_clear_search_results, self, 0);
-#line 393 "tracker-result-grid.c"
+#line 375 "tracker-result-grid.c"
 	}
 	g_object_notify ((GObject *) self, "Query");
 }
 
 
-#line 159 "tracker-result-grid.gs"
+#line 155 "tracker-result-grid.gs"
 static void _tracker_result_grid_ActivateUri_gtk_icon_view_item_activated (GtkIconView* _sender, const GtkTreePath* path, gpointer self) {
-#line 401 "tracker-result-grid.c"
+#line 383 "tracker-result-grid.c"
 	tracker_result_grid_ActivateUri (self, path);
 }
 
@@ -405,9 +387,9 @@ static void _tracker_result_grid_ActivateUri_gtk_icon_view_item_activated (GtkIc
 static void _lambda4_ (TrackerResultGrid* self) {
 	GtkTreePath* path;
 	path = tracker_result_grid_GetSelectedPath (self);
-#line 107 "tracker-result-grid.gs"
+#line 103 "tracker-result-grid.gs"
 	g_signal_emit_by_name (self, "selection-changed", path);
-#line 411 "tracker-result-grid.c"
+#line 393 "tracker-result-grid.c"
 	_gtk_tree_path_free0 (path);
 }
 
@@ -422,20 +404,20 @@ static void _lambda5_ (GdkDragContext* context, GtkSelectionData* data, guint in
 	g_return_if_fail (context != NULL);
 	g_return_if_fail (data != NULL);
 	uri = g_strdup (tracker_result_grid_GetSelectedUri (self));
-#line 112 "tracker-result-grid.gs"
+#line 108 "tracker-result-grid.gs"
 	if (uri != NULL) {
-#line 428 "tracker-result-grid.c"
+#line 410 "tracker-result-grid.c"
 		char** _tmp0_;
 		gint s_size;
 		gint s_length1;
 		char** s;
 		char* _tmp1_;
 		s = (_tmp0_ = g_new0 (char*, 1 + 1), s_length1 = 1, s_size = s_length1, _tmp0_);
-#line 114 "tracker-result-grid.gs"
+#line 110 "tracker-result-grid.gs"
 		s[0] = (_tmp1_ = g_strdup (uri), _g_free0 (s[0]), _tmp1_);
-#line 115 "tracker-result-grid.gs"
+#line 111 "tracker-result-grid.gs"
 		gtk_selection_data_set_uris (data, s);
-#line 439 "tracker-result-grid.c"
+#line 421 "tracker-result-grid.c"
 		s = (_vala_array_free (s, s_length1, (GDestroyNotify) g_free), NULL);
 	}
 	_g_free0 (uri);
@@ -459,39 +441,39 @@ static GObject * tracker_result_grid_constructor (GType type, guint n_construct_
 	{
 		GtkListStore* _tmp0_;
 		GtkIconView* _tmp1_;
-#line 87 "tracker-result-grid.gs"
+#line 83 "tracker-result-grid.gs"
 		g_object_set ((GtkScrolledWindow*) self, "hscrollbar-policy", GTK_POLICY_AUTOMATIC, NULL);
-#line 88 "tracker-result-grid.gs"
+#line 84 "tracker-result-grid.gs"
 		g_object_set ((GtkScrolledWindow*) self, "vscrollbar-policy", GTK_POLICY_AUTOMATIC, NULL);
-#line 89 "tracker-result-grid.gs"
+#line 85 "tracker-result-grid.gs"
 		gtk_scrolled_window_set_shadow_type ((GtkScrolledWindow*) self, GTK_SHADOW_ETCHED_OUT);
-#line 91 "tracker-result-grid.gs"
+#line 87 "tracker-result-grid.gs"
 		self->store = (_tmp0_ = gtk_list_store_new ((gint) RESULT_COLUMNS_NumOfCols, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_STRING, NULL), _g_object_unref0 (self->store), _tmp0_);
-#line 96 "tracker-result-grid.gs"
+#line 92 "tracker-result-grid.gs"
 		self->iconview = (_tmp1_ = g_object_ref_sink ((GtkIconView*) gtk_icon_view_new_with_model ((GtkTreeModel*) self->store)), _g_object_unref0 (self->iconview), _tmp1_);
-#line 97 "tracker-result-grid.gs"
+#line 93 "tracker-result-grid.gs"
 		gtk_icon_view_set_pixbuf_column (self->iconview, (gint) RESULT_COLUMNS_Icon);
-#line 98 "tracker-result-grid.gs"
+#line 94 "tracker-result-grid.gs"
 		gtk_icon_view_set_text_column (self->iconview, (gint) RESULT_COLUMNS_DisplayName);
-#line 99 "tracker-result-grid.gs"
+#line 95 "tracker-result-grid.gs"
 		gtk_icon_view_set_selection_mode (self->iconview, GTK_SELECTION_BROWSE);
-#line 100 "tracker-result-grid.gs"
+#line 96 "tracker-result-grid.gs"
 		gtk_icon_view_enable_model_drag_source (self->iconview, GDK_BUTTON1_MASK | GDK_BUTTON2_MASK, targets, G_N_ELEMENTS (targets), (GDK_ACTION_COPY | GDK_ACTION_MOVE) | GDK_ACTION_ASK);
-#line 101 "tracker-result-grid.gs"
+#line 97 "tracker-result-grid.gs"
 		gtk_icon_view_set_item_width (self->iconview, 150);
-#line 102 "tracker-result-grid.gs"
+#line 98 "tracker-result-grid.gs"
 		gtk_icon_view_set_row_spacing (self->iconview, 10);
-#line 103 "tracker-result-grid.gs"
+#line 99 "tracker-result-grid.gs"
 		g_signal_connect_object (self->iconview, "item-activated", (GCallback) _tracker_result_grid_ActivateUri_gtk_icon_view_item_activated, self, 0);
-#line 105 "tracker-result-grid.gs"
+#line 101 "tracker-result-grid.gs"
 		g_signal_connect_object (self->iconview, "selection-changed", (GCallback) __lambda4__gtk_icon_view_selection_changed, self, 0);
-#line 110 "tracker-result-grid.gs"
+#line 106 "tracker-result-grid.gs"
 		g_signal_connect_object ((GtkWidget*) self->iconview, "drag-data-get", (GCallback) __lambda5__gtk_widget_drag_data_get, self, 0);
-#line 117 "tracker-result-grid.gs"
+#line 113 "tracker-result-grid.gs"
 		gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->iconview);
-#line 118 "tracker-result-grid.gs"
+#line 114 "tracker-result-grid.gs"
 		gtk_widget_show_all ((GtkWidget*) self);
-#line 495 "tracker-result-grid.c"
+#line 477 "tracker-result-grid.c"
 	}
 	return obj;
 }
