@@ -169,7 +169,7 @@ sparql_query_cb (GPtrArray *result,
 	priv = TRACKER_WRITEBACK_CONSUMER_GET_PRIVATE (consumer);
 	data = g_queue_pop_head (priv->process_queue);
 
-	if (result && result->len > 0) {
+	if (!error && result && result->len > 0) {
 		GHashTableIter iter;
 		gpointer key, value;
 		GStrv rdf_types;
@@ -227,15 +227,12 @@ process_queue_cb (gpointer user_data)
 		return FALSE;
 	}
 
-	/* We use IE = DO, so we can optimize using nie:isStoredAs instead of
-	 * nie:url here. Please change this when we change that decision. */
-
-	query = g_strdup_printf ("SELECT ?url ?predicate ?object {"
-	                         "  <%s> ?predicate ?object ;"
-	                         "  nie:isStoredAs ?url ."
+	query = g_strdup_printf ("SELECT ?url '%s' ?predicate ?object {"
+	                         "  <%s> ?predicate ?object ; "
+	                         "       nie:url ?url ."
 	                         "  ?predicate tracker:writeback true "
 	                         "}",
-	                         data->subject);
+	                         data->subject, data->subject);
 
 	tracker_resources_sparql_query_async (priv->client,
 	                                      query,
